const helpers = require('../../helpers/helpers')
const teamHelper = require('../../helpers/Team')
const mailService = require('pubsweet-component-mail-service')
const logger = require('@pubsweet/logger')
const config = require('config')
const userHelper = require('../../helpers/User')
const collectionHelper = require('../../helpers/Collection')
const authsomeHelper = require('../../helpers/authsome')

const statuses = config.get('statuses')
module.exports = models => async (req, res) => {
  const { collectionId, invitationId } = req.params
  try {
    const collection = await models.Collection.find(collectionId)
    const authsome = authsomeHelper.getAuthsome(models)
    const target = {
      collection,
      path: req.route.path,
    }
    const canDelete = await authsome.can(req.user, 'DELETE', target)
    if (!canDelete)
      return res.status(403).json({
        error: 'Unauthorized.',
      })
    const invitation = await collection.invitations.find(
      invitation => invitation.id === invitationId,
    )
    if (invitation === undefined) {
      res.status(404).json({
        error: `Invitation ${invitationId} not found`,
      })
      return
    }
    const team = await teamHelper.getTeamByGroupAndCollection(
      collectionId,
      invitation.role,
      models.Team,
    )

    collection.invitations = collection.invitations.filter(
      inv => inv.id !== invitation.id,
    )
    if (invitation.role === 'handlingEditor') {
      collection.status = 'submitted'
      collection.visibleStatus = statuses[collection.status].private
      delete collection.handlingEditor
    } else if (invitation.role === 'reviewer') {
      await collectionHelper.updateReviewerCollectionStatus(collection)
    }
    await collection.save()
    await teamHelper.removeTeamMember(team.id, invitation.userId, models.Team)
    const user = await models.User.find(invitation.userId)
    user.teams = user.teams.filter(userTeamId => team.id !== userTeamId)
    await user.save()
    try {
      if (invitation.role === 'handlingEditor') {
        await mailService.sendSimpleEmail({
          toEmail: user.email,
          emailType: 'revoke-handling-editor',
        })
      } else if (invitation.role === 'reviewer') {
        await userHelper.setupReviewerUnassignEmail({
          UserModel: models.User,
          FragmentModel: models.Fragment,
          collection,
          user,
          mailService,
        })
      }

      return res.status(200).json({})
    } catch (e) {
      logger.error(e.message)
      return res.status(500).json({ error: 'Email could not be sent.' })
    }
  } catch (e) {
    const notFoundError = await helpers.handleNotFoundError(e, 'collection')
    return res.status(notFoundError.status).json({
      error: notFoundError.message,
    })
  }
}
