const querystring = require('querystring')
const fs = require('fs')
const handlebars = require('handlebars')

const createUrl = (baseUrl, slug, queryParams = null) =>
  !queryParams
    ? `${baseUrl}${slug}`
    : `${baseUrl}${slug}?${querystring.encode(queryParams)}`

const getEmailBody = (emailType, replacements) => {
  handlePartial('header', { previewText: replacements.previewText })
  handlePartial('footer')
  const buttonContext = {
    url: replacements.url,
    buttonText: replacements.buttonText,
  }
  handlePartial('mainButton', buttonContext)

  const bodyContext = {
    headline: replacements.headline,
    paragraph: replacements.paragraph,
  }
  handlePartial('mainBody', bodyContext)

  return getMainTemplate(emailType, replacements)
}

const getNotificationBody = (emailType, replacements) => {
  handlePartial('notificationHeader', { previewText: replacements.previewText })
  handlePartial('footer')

  handlePartial('signature', {
    signatureName: replacements.signatureName,
  })

  if (replacements.detailsUrl !== undefined)
    handlePartial('manuscriptDetailsLink', {
      beforeAnchor: replacements.beforeAnchor,
      detailsUrl: replacements.detailsUrl,
      afterAnchor: replacements.afterAnchor,
    })

  handlePartial('notificationBody', {
    intro: replacements.intro,
    paragraph: replacements.paragraph,
    hasLink: replacements.hasLink,
  })

  return getMainTemplate(emailType, replacements)
}

const getInvitationBody = (emailType, replacements) => {
  handlePartial('invitationHeader', { previewText: replacements.previewText })
  handlePartial('footer')

  const upperContext = {
    intro: replacements.intro,
    upperContent: replacements.upperContent,
  }
  handlePartial('invitationUpperContent', upperContext)

  const buttonsContext = {
    declineUrl: replacements.declineUrl,
    agreeUrl: replacements.agreeUrl,
  }
  handlePartial('invitationButtons', buttonsContext)

  const manuscriptContext = {
    manuscriptText: replacements.manuscriptText,
    title: replacements.title,
    authors: replacements.authors,
    abstract: replacements.abstract,
  }
  handlePartial('manuscriptData', manuscriptContext)

  const signatureContext = {
    signatureName: replacements.signatureName,
    signatureEmail: replacements.signatureEmail,
  }
  handlePartial('signature', signatureContext)

  const lowerContext = {
    baseUrl: replacements.baseUrl,
    lowerContent: replacements.lowerContent,
    signature: replacements.signature,
  }
  handlePartial('invitationLowerContent', lowerContext)

  return getMainTemplate(emailType, replacements)
}

const readFile = path =>
  fs.readFileSync(path, { encoding: 'utf-8' }, (err, file) => {
    if (err) {
      throw err
    } else {
      return file
    }
  })

const handlePartial = (partialName, context = {}) => {
  let partial = readFile(
    `${__dirname}/../templates/partials/${partialName}.hbs`,
  )
  const template = handlebars.compile(partial)
  partial = template(context)
  handlebars.registerPartial(partialName, partial)
}

const getMainTemplate = (fileName, context) => {
  const htmlFile = readFile(`${__dirname}/../templates/${fileName}.html`)
  const htmlTemplate = handlebars.compile(htmlFile)
  const htmlBody = htmlTemplate(context)
  return htmlBody
}

const getExpectedDate = (timestamp, daysExpected) => {
  const date = new Date(timestamp)
  let expectedDate = date.getDate() + daysExpected
  date.setDate(expectedDate)
  expectedDate = date.toLocaleDateString('en-US', {
    day: 'numeric',
    month: 'long',
    year: 'numeric',
  })
  return expectedDate
}

module.exports = {
  createUrl,
  getEmailBody,
  getExpectedDate,
  getNotificationBody,
  getInvitationBody,
}
