import React from 'react'
import { get } from 'lodash'
import { connect } from 'react-redux'
import { AbstractEditor } from 'xpub-edit'
import { required } from 'xpub-validators'
import { compose, withHandlers } from 'recompose'
import { withModal } from 'pubsweet-component-modal/src/components'
import {
  Icon,
  Button,
  Spinner,
  ErrorText,
  RadioGroup,
  ValidatedField,
} from '@pubsweet/ui'
import {
  reduxForm,
  isSubmitting,
  getFormValues,
  change as changeForm,
} from 'redux-form'

import {
  Row,
  ModalRoot,
  CloseIcon,
  ModalTitle,
  ActionButton,
  CustomRadioGroup,
  ButtonsContainer,
  SpinnerContainer,
} from '../atoms'

const options = [
  {
    value: 'publish',
    label: 'Confirm recommendation to Publish',
  },
  {
    value: 'reject',
    label: 'Confirm recommendation to Reject',
  },
  {
    value: 'return',
    label: 'Return manuscript with comments',
  },
]

const MakeDecision = ({ showDecisionModal }) => (
  <ActionButton onClick={showDecisionModal}>Make Decision</ActionButton>
)

// To be removed
const sleep = ms => new Promise(resolve => setTimeout(resolve, ms))

const DecisionModal = compose(
  connect(
    state => ({
      isSubmitting: isSubmitting('makeDecision')(state),
      decision: get(getFormValues('makeDecision')(state), 'decision'),
    }),
    { changeForm, getFormValues },
  ),
  reduxForm({
    form: 'makeDecision',
    onSubmit: (values, dispatch, { isSubmitting }) =>
      sleep(1000).then(() => {
        // TODO: link to backend
        window.alert(`You submitted:\n\n${JSON.stringify(values, null, 2)}`)
      }),
  }),
  withHandlers({
    selectOption: ({ changeForm, formValues }) => value => {
      changeForm('makeDecision', 'decision', value)
    },
  }),
)(
  ({
    error,
    decision,
    hideModal,
    handleSubmit,
    selectOption,
    isSubmitting,
  }) => (
    <ModalRoot>
      <CloseIcon data-test="icon-modal-hide" onClick={hideModal}>
        <Icon primary>x</Icon>
      </CloseIcon>

      <ModalTitle>Manuscript Decision</ModalTitle>
      <CustomRadioGroup>
        <RadioGroup
          isRequired
          name="decision"
          onChange={selectOption}
          options={options}
        />
      </CustomRadioGroup>
      {decision === 'return' && (
        <Row>
          <ValidatedField
            component={AbstractEditor}
            name="comment"
            placeholder="Return with comment (required)"
            title="Return with comment (required)"
            validate={[required]}
          />
        </Row>
      )}
      {error && <ErrorText>{error}</ErrorText>}
      <ButtonsContainer>
        <Button data-test="button-modal-hide" onClick={hideModal}>
          Cancel
        </Button>
        {isSubmitting ? (
          <SpinnerContainer>
            <Spinner size={4} />
          </SpinnerContainer>
        ) : (
          <Button
            data-test="button-modal-confirm"
            onClick={handleSubmit}
            primary
          >
            SEND REQUEST
          </Button>
        )}
      </ButtonsContainer>
    </ModalRoot>
  ),
)

export default compose(
  connect(null, null),
  withModal({
    modalKey: 'EiCMakeDecision',
    modalComponent: DecisionModal,
  }),
  withHandlers({
    showDecisionModal: ({ showModal }) => () => {
      showModal()
    },
  }),
)(MakeDecision)
