import React from 'react'
import { compose } from 'recompose'
import { connect } from 'react-redux'
import styled from 'styled-components'
import { th, Icon } from '@pubsweet/ui'
import ZipFiles from 'pubsweet-components-faraday/src/components/Files/ZipFiles'
import { Recommendation } from 'pubsweet-components-faraday/src/components/MakeRecommendation'

import { MakeDecision } from './'
import { canMakeRecommendation } from '../../../component-faraday-selectors/src'

const SideBarActions = ({
  project,
  version,
  currentUserIs,
  canMakeRecommendation,
}) => (
  <Root>
    {currentUserIs('adminEiC') && <MakeDecision />}
    {canMakeRecommendation && (
      <Recommendation
        collectionId={project.id}
        fragmentId={version.id}
        modalKey={`recommend-${version.id}`}
      />
    )}
    <ZipFiles
      archiveName={`ID-${project.customId}`}
      collectionId={project.id}
      fragmentId={version.id}
    >
      <ClickableIcon>
        <Icon>download</Icon>
      </ClickableIcon>
    </ZipFiles>
  </Root>
)

export default compose(
  connect((state, { project }) => ({
    canMakeRecommendation: canMakeRecommendation(state, project),
  })),
)(SideBarActions)

// #region styled-components
const Root = styled.div`
  border-bottom: ${th('borderDefault')};
  display: flex;
  justify-content: space-between;
  padding: ${th('subGridUnit')};
`

const ClickableIcon = styled.div`
  margin: 0 ${th('subGridUnit')};
  &:hover {
    opacity: 0.7;
  }
`
// #endregion
