const logger = require('@pubsweet/logger')
const helpers = require('../../helpers/helpers')

module.exports = models => async (req, res) => {
  // TO DO: add authsome
  const { collectionId, userId } = req.params
  const {
    isSubmitting,
    isCorresponding,
    firstName,
    lastName,
    affiliation,
  } = req.body

  if (!helpers.checkForUndefinedParams(isSubmitting, isCorresponding)) {
    res.status(400).json({ error: 'Missing parameters' })
    logger.error('some parameters are missing')
    return
  }

  try {
    let collection = await models.Collection.find(collectionId)
    if (collection.authors === undefined) {
      return res.status(400).json({
        error: 'Collection does not have any authors',
      })
    }
    const user = await models.User.find(userId)
    const matchingAuthor = collection.authors.find(
      author => author.userId === user.id,
    )
    if (matchingAuthor === undefined) {
      return res.status(400).json({
        error: 'Collection and user do not match',
      })
    }
    user.firstName = firstName
    user.lastName = lastName
    user.affiliation = affiliation
    await user.save()
    matchingAuthor.isSubmitting = isSubmitting
    matchingAuthor.isCorresponding = isCorresponding
    collection = await collection.save()
    res.status(200).json(collection)
  } catch (e) {
    const notFoundError = await helpers.handleNotFoundError(e, 'item')
    return res.status(notFoundError.status).json({
      error: notFoundError.message,
    })
  }
}
