import React from 'react'
import { withProps } from 'recompose'
import { Route, Switch } from 'react-router-dom'
import { AuthenticatedComponent } from 'pubsweet-client'
import Login from 'pubsweet-component-login/LoginContainer'
import Signup from 'pubsweet-component-signup/SignupContainer'

import { Wizard } from 'pubsweet-component-wizard/src/components'
import { ManuscriptPage } from 'pubsweet-component-manuscript/src/components'
import DashboardPage from 'pubsweet-components-faraday/src/components/Dashboard'
import {
  NotFound,
  ConfirmationPage,
  ErrorPage,
} from 'pubsweet-components-faraday/src/components/UIComponents/'
import {
  AdminDashboard,
  AdminUsers,
  AdminRoute,
} from 'pubsweet-components-faraday/src/components/Admin'
import AddEditUser from 'pubsweet-components-faraday/src/components/Admin/AddEditUser'
import {
  SignUpInvitationPage,
  ReviewerSignUp,
} from 'pubsweet-components-faraday/src/components/SignUp'

import FaradayApp from './FaradayApp'

const LoginPage = withProps({ passwordReset: false })(Login)

const PrivateRoute = ({ component: Component, ...rest }) => (
  <Route
    {...rest}
    render={props => (
      <AuthenticatedComponent>
        <Component {...props} />
      </AuthenticatedComponent>
    )}
  />
)

const Routes = () => (
  <FaradayApp>
    <Switch>
      <Route component={LoginPage} exact path="/login" />
      <Route component={Signup} exact path="/signup" />
      <PrivateRoute component={DashboardPage} exact path="/" />
      <PrivateRoute
        component={ConfirmationPage}
        exact
        path="/confirmation-page"
      />
      <AdminRoute component={AdminDashboard} exact path="/admin" />
      <AdminRoute component={AdminUsers} exact path="/admin/users" />
      <AdminRoute component={AddEditUser} exact path="/admin/users/add" />
      <AdminRoute
        component={AddEditUser}
        exact
        path="/admin/users/edit/:userId"
      />
      <PrivateRoute
        component={Wizard}
        exact
        path="/projects/:project/versions/:version/submit"
      />
      <Route component={SignUpInvitationPage} exact path="/invite" />
      <Route component={ReviewerSignUp} exact path="/invite-reviewer" />
      <PrivateRoute
        component={ManuscriptPage}
        exact
        path="/projects/:project/versions/:version/details"
      />

      <Route component={ErrorPage} exact path="/error-page" />
      <Route component={NotFound} />
    </Switch>
  </FaradayApp>
)

export default Routes
